#!/usr/bin/env bash

# provision.sh v. 2.22
# (Changelog: https://teampasswordmanager.com/docs/install-vagrant-virtualbox/#changelog)
#
# Ferran Barba, May 2019
# info@teampasswordmanager.com
# https://teampasswordmanager.com
#
# Vagrant shell provisioning file to install the required software to execute Team Password Manager
#
# It installs: 
#	- Some linux utilities: unzip
#	- Apache 2 
#	- MySQL Server 5.7
#	- PHP 7.2
#	- Ioncube Loader v. 10.3.4
#	- Team Password Manager (latest version)
#	- Firewall (http/s, ssh, ntp)
#	- fail2ban

# Parameters from the Vagrantfile

PARAM_HOSTNAME=$1
PARAM_SERVER_SWAP=$2
PARAM_SERVER_TIMEZONE=$3
PARAM_MYSQL_ROOT_PASSWORD=$4
PARAM_PHP_TIMEZONE=$5
PARAM_TPM_DATABASE=$6
PARAM_TPM_USER=$7
PARAM_TPM_PASSWORD=$8

# Let's go

echo "* TPM PROVISION BEGIN $(date)" 
echo "**********************************************"

echo "* Parameters:"
echo "*   Hostname: $PARAM_HOSTNAME" 
echo "*   Server swap: $PARAM_SERVER_SWAP" 
echo "*   Server timezone: $PARAM_SERVER_TIMEZONE" 
echo "*   PHP timezone: $PARAM_PHP_TIMEZONE" 

# ***************** BASE INSTALLATION ***************** 

# Update
echo "* Updating repositories" 
sudo apt-get update 

# Set timezone 
echo "* Setting Timezone to $PARAM_SERVER_TIMEZONE"
sudo timedatectl set-timezone $PARAM_SERVER_TIMEZONE

# Turn on time sync
echo "* Turning on time sync"
sudo timedatectl set-ntp off
sudo timedatectl set-ntp on 

# Some utilities (the other utilities installed in previous versions are already installed
# curl software-properties-common acl htop)
echo "* Installing unzip" 
sudo apt-get install -qq unzip

# Set up swap
shopt -s nocasematch
if [[ ! -z $PARAM_SERVER_SWAP && ! $PARAM_SERVER_SWAP =~ false && $PARAM_SERVER_SWAP =~ ^[0-9]*$ ]]; then
	echo "* Setting up Swap ($PARAM_SERVER_SWAP MB)" 

	# Create the Swap file
	sudo fallocate -l "$PARAM_SERVER_SWAP"M /swapfile

	# Set the correct Swap permissions
	sudo chmod 600 /swapfile

	# Setup Swap space
	sudo mkswap /swapfile

    # Enable Swap space
    sudo swapon /swapfile

	# Make the Swap file permanent
	echo "/swapfile   none    swap    sw    0   0"

	# Add some swap settings:
	# vm.swappiness=10: Means that there wont be a Swap file until memory hits 90% useage
	# vm.vfs_cache_pressure=50: read http://rudd-o.com/linux-and-free-software/tales-from-responsivenessland-why-linux-feels-slow-and-how-to-fix-that
	printf "vm.swappiness=10\nvm.vfs_cache_pressure=50" | tee -a /etc/sysctl.conf && sysctl -p
fi
shopt -u nocasematch

# ***************** REQUIRED SOFTWARE: APACHE, MYSQL, PHP ***************** 

# Apache (html files in /var/www/html)
echo "**********************************************"
echo "* Installing and configuring Apache" 
sudo apt-get install -qq apache2  

# ServerName 
echo "ServerName $PARAM_HOSTNAME" | sudo tee /etc/apache2/conf-available/fqdn.conf && sudo a2enconf fqdn 

# SSL self signed certificate
sudo make-ssl-cert generate-default-snakeoil --force-overwrite
sudo a2enmod ssl
sudo a2ensite default-ssl.conf
sudo systemctl restart apache2

# MYSQL Server
echo "**********************************************"
echo "* Installing and configuring MySQL Server" 

export DEBIAN_FRONTEND="noninteractive"

# Set username and password to 'root'
sudo debconf-set-selections <<< "mysql-server mysql-server/root_password password $PARAM_MYSQL_ROOT_PASSWORD"
sudo debconf-set-selections <<< "mysql-server mysql-server/root_password_again password $PARAM_MYSQL_ROOT_PASSWORD"

# Install MySQL Server
sudo apt-get install -qq mysql-server-5.7 

# Create database and user for Team Password Manager
MYSQL=`which mysql`
Q1="CREATE DATABASE $PARAM_TPM_DATABASE CHARACTER SET utf8 COLLATE utf8_unicode_ci;"
Q2="GRANT ALL PRIVILEGES ON $PARAM_TPM_DATABASE.* to '$PARAM_TPM_USER' IDENTIFIED BY '$PARAM_TPM_PASSWORD';"
Q3="FLUSH PRIVILEGES;" 
SQL="${Q1}${Q2}${Q3}"
$MYSQL -uroot -p$PARAM_MYSQL_ROOT_PASSWORD -e "$SQL"  

# PHP
echo "**********************************************"
echo "* Installing PHP" 
sudo add-apt-repository -y ppa:ondrej/php
sudo apt-get update 
sudo apt-get install -qq php7.2 php7.2-cli php7.2-mysql php7.2-mbstring php7.2-ldap php7.2-curl php7.2-gd

# ***************** IONCUBE ***************** 

# Ioncube Loader
echo "**********************************************"
echo "* Installing Ioncube Loader" 
cd /usr/lib/php/20170718
sudo wget --quiet https://teampasswordmanager.com/assets/download/vagrant/ioncube/10.3.4/64/ioncube_loader_lin_7.2.so
echo "zend_extension = /usr/lib/php/20170718/ioncube_loader_lin_7.2.so" | sudo tee /etc/php/7.2/apache2/conf.d/01-ioncube.ini

# Restart Apache
sudo systemctl restart apache2

# ***************** TEAM PASSWORD MANAGER ***************** 

# Team Password Manager
echo "**********************************************"
echo "* Downloading and preparing Team Password Manager for installation" 

# Get the latest version of Team Password Manager and uncompress it in /var/www/html/teampasswordmanager
LATEST_TPM_VERSION=$(curl --silent https://teampasswordmanager.com/latest-version)
cd /vagrant
sudo wget --quiet https://teampasswordmanager.com/assets/download/teampasswordmanager_"$LATEST_TPM_VERSION".zip  
sudo unzip teampasswordmanager_"$LATEST_TPM_VERSION".zip -d /var/www/html  
sudo mv /var/www/html/teampasswordmanager_"$LATEST_TPM_VERSION"/ /var/www/html/teampasswordmanager  
rm teampasswordmanager_"$LATEST_TPM_VERSION".zip # delete downloaded file

# Make site files have user/group www-data (the user that Apache uses)
sudo chown -R www-data:www-data /var/www

# Make user vagrant member of www-data group and make the www folder writable by www-data group too
# Uncomment this if you want the vagrant user to be able to write to /var/www
#sudo usermod -a -G www-data vagrant
#sudo chmod -R 775 /var/www

# Set database, user and password in config.php
sudo sed -i "s/'database'/'tpm_database'/" /var/www/html/teampasswordmanager/config.php
sudo sed -i "s/'user'/'tpm_user'/" /var/www/html/teampasswordmanager/config.php
sudo sed -i "s/'password'/'tpm_password'/" /var/www/html/teampasswordmanager/config.php

# ***************** FIREWALL AND FAIL2BAN *****************

# Firewall (http/s, ssh and ntp)
echo "**********************************************"
echo "* Setting up the firewall" 

sudo ufw allow OpenSSH
sudo ufw allow 'Apache Full'
sudo ufw allow ntp 
sudo ufw --force enable

# fail2ban (prevents dictionary attacks on the server)
echo "* Installing fail2ban"

sudo apt-get install -qq fail2ban
sudo cp /etc/fail2ban/jail.conf /etc/fail2ban/jail.local


# ***************** ENDING *****************

# **** End
echo "**********************************************"
echo "* System is installed" 
echo "* Open your browser and go to:"
echo "*    http://localhost:8080/teampasswordmanager/index.php/install"
echo "* to finish installing Team Password Manager" 
echo "**********************************************"
echo "* TPM PROVISION END $(date)" 
